## Templating

Another pre-delivered feature of the Pretzel suite is templating. This is done
with the excellent [Liquid templating language.](https://github.com/shopify/liquid)

The purpose of this document is to show how to use the Pretzel suite with Liquid,
not to explain the templating language itself. For that, visit the link above and
look through the wiki to get a basic grasp.

Pretzel projects once again take a convention-over-configuration approach to templating.
All your templates must be stored in a `views` that resides next to your application file.

Inside the view folder you have free reign as to how to structure your templates with
one notable exceptions: *Liquid partials go in the `views/includes` folder*. As a fair
hint: Remember that all partials need to be named with an underscore in front. This
has caused me some headaches before.

Now for the folder structure:

```
  app
   |
   |- application.rb
   |- config.ru
   |- views
        |- includes
          |- _partial.liquid
        |- your_template.liquid
```

### Preparing your application

The code side of things is even simpler. The templating functionality is provided
in an extension: `Pretzel::Extension::Templating`. As with other extensions you have
to include it before using it, like so:

```ruby
  require "pretzel/core"
  require "pretzel/ext/templating"

  class Application < Pretzel::Core
    extends Pretzel::Extension::Templating
  end
```

### Using basic templates

The extension exposes a single function: `render`, which is used like this:

```ruby
  Application.get "/" do
    render :index
  end
```

The template name is passed as first argument. It can be either a symbol or a string.
It *must* not contain the file extension and is always rooted in the views folder.

Here are some more examples

```ruby
  render :index # => views/index.liquid
  render "subfolder/my_template" # => views/subfolder/my_template.liquid
  render :"subfolder/other_template" # => views/subfolder/other_template.liquid
```

### Templates and locals

Most templates aren't satisfied with simple rendering, and usually you want to pass
a bunch of variables (so called `locals`) in. The render function has that covered:

```ruby
  Application.get "/" do
    render :index, {
      "name" => "Richard"
    }
  end
```

The locals are always passed in as a simple Ruby Hash. For more information on locals,
see the Liquid documentation!

### Layouts

Many applications have some sort of HTML structure that they want to persist across
various views. This is usually called a layout or a theme. The Pretzel suite has
this integrated too. 

Whenever you call `render` on a template, the framework will look for a file in your
`views` folder called `layout.liquid`. This template exposes a special variable 
called `template`. An example layout may look like this:

```html
  <!DOCTYPE html>
  <html lang="en">
    <head>
      <meta charset="UTF-8">
      <title>Document</title>
    </head>
    <body>
      {{ template }}    
    </body>
  </html>
```

The part where you output template is, shockingly, the place where your template will
appear. Cool, huh?

#### Deactivating the layout

If there is no `layout.liquid` file, the Pretzel suite will just not apply a layout.
In case you have created a layout but you want to deactivate it for a certain route,
you just need to pass `"layout" => false` into the locals like so:

```ruby
  Application.get "/no-layout" do
    render :some_template, {
      "layout" => false
    }
  end
```

This will supress the layout rendering for this template.