## Pretzel extensions

While Rack middleware is great for low-level extensions, sometimes you 
may want to extend the Pretzel suite on a somewhat higher level.

Maybe your extension requires some custom routes or you want to include
some Liquid extensions? This is the reason for the `extends` method.

It is used like this:

```ruby
  require "pretzel/core"

  class Application < Pretzel::Core
    extends Pretzel::Extension::Test
  end
```

And you're good to go. 

## Structure

Each extension consists of two parts: `App` and `Route`.
Both are submodules of your extension. The structure looks like this.

```ruby
  module Pretzel::Extension

      module Test
        module App
          # Code-y awesomeness
        end

        module Route
          # Code-y awesomeness
        end
      end

  end
```

## Where to declare things

The `App` and `Route` modules are rather aptly named. Any methods declared in
the `App` module will be available in the class declaration like this:

```ruby
  class Application < Pretzel::Core
    extends Your::Extension
    your_method
  end
```

On the other side, any methods declared in the `Route` module are available in
the route controllers like this:

```ruby
  class Application < Pretzel::Core
    extends Your::Extension
  end

  Application.get "/" do
    your_extension
  end
```

### Custom extension routes

The `Route` module is also where you want to declare your custom routes.
This can be done by using Ruby's `self.included` method to your advantage:

```ruby
  module Pretzel::Extension

      module Test
        module Route
          
          def self.included(application)
            application.get "/ext/index" do
              "Extensions!"
            end
          end

        end
      end

  end
```

### Extending Liquid

If you want to keep your application clean and modular, you can also use the
`Route` module to extend Liquid. You can read all about [the subject matter here.](https://github.com/Shopify/liquid/wiki/Liquid-for-Programmers#extending-liquid)

An example Liquid extension may look like this:

```ruby
  
  module Pretzel::Extension

      module Test
        module Route
          module TestFilter
            def reverse(string)
              string.reverse!
            end
          end
          
          Liquid::Template.register_filter(TestFilter)
        end
      end

  end
```

It can then be included in your application with `extends` and you're ready
to use the filter in your templates. (Of course you can also create blocks and tags.)